''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' Battery Management Program for Caravan 12V System                            '
' CVANBMON.BAS                                                                 '
' Copyright Doug Pankhurst May 2012                                            '
' additional routines copyright Geoff Graham 2012                              '
' Written for MMBasic V4.1, copyright Geoff Graham 2012                        '
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' File Version:    1.00DB   10 Jan 2013
' - debug or simulator version.

' This program monitors a 12V Battery System, monitoring the battery voltage,
' the amount of charge supplied into the system via an input current shunt, the
' amount of charge used by the system via a load current shunt and calculates
' the charge/discharge state of the battery by comparing current in versus
' current out. It also estimates time to 30% capacity based on battery voltage
' and current discharge rate and provides a warning.
' Real time data is displayed on VGA display as well as historical
' data recorded to a log file on the SD card for later analysis.

' The program is highly commented - because of this, it will not run on a
' Colour MM due to memory space limitations. Use Tassie Jim's excellent MMEdit
' to strip comments to run on Colour MM.  It will run as is on mono MM
' (Duinomite Mega and Dontronics SM1)

' Utilises a range of subroutines and defined funtions based on software from
' Doug Pankhurst, Geoff Graham, Tom Pankhurst, TassieJim and also from programs
' provided in the MMBASIC Software library, authors unknown.
' This code is free for anyone to use or modify as long as they acknowledge it's
' source and credit the contributors above.
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' A general synopsis of the program is as follows:-
' Define and initialise variables and constants;
' Get and set current date and time;
' Initialise I/O pins used
' Initialise display static data
' Initialise interrupt routine to handle 2 second interrupt for data scan
' Enter main loop; wait for interrupt then handle 2 second, 1 minute, hourly
'  and end of day occurrences. Interrupt provides battery volts, solar volts,
'  input current and load current.
' 1 minute flag allows averaging battery and solar volts, input and load
'  currents each minute. Also plugs new averages into log array and updates
'  both the bar graph and the running graph displays
' Hourly flag writes 1 hours worth of average data from array to log file
' End of day flag closes out log file and creates new log file
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' Main Entry to Program                                                       '
'
EntryPoint:  SetTick 0,0     ' Disable timer interrupt
'
' Constants
TRUE = 1
FALSE = 0
HIGH = 1
LOW = 0

' Global Variables
BVolts = 12.8          ' Battery Voltage read from Pin 4. Read as one sixth of
                       ' real voltage via 6:1 resistive divider. Eg. 2V
                       ' reading equates to a battery voltage of 12V

Dim BVArray(29)        ' Battary Volts 30 element array - 1 each 2 secs

BatCharge = 100        ' Battery state of charge as a percentage where
                       ' 12.75V or higher is 100%, down to 12.05V being 35%
                       ' - this is considered the danger level for discharge

AverageBV=12.8         ' Average battery voltage over last 60 seconds

Dim BVAArray(59)       ' 1 hours worth of average bat volts
                       ' caculated each minute

DeltaBV = 0.1          ' Battery voltage variation over 60 seconds. The
                       ' difference between the first and the last used to
                       ' calculate the "Time to 30% discharge point"

Time2Discharge = 20    ' Time to 30% discharge

SVolts = 20            ' Solar Array voltage from Pin 3. Read as one tenth
                       ' of real voltage by 10:1 resistive divider. Eg. 2.0V
                       ' equates to 20V at the solar panel

Dim SVArray(29)        ' Solar Volts 30 element array - 1 each 2 secs

AverageSV = 21         ' Average Solar Panel volts over last 60 seconds

Dim SVAArray(59)       ' 1 hours worth of average solar volts calculated
                       ' each minute

InputCurrent = 18      ' Charging current into battery/load read from Pin 1
                       ' Note: 2.5V reading equates to 50mA across the shunt
                       ' which in turn equates to 50A through shunt.
                       ' This can be both battery charging current
                       ' and load current.

Dim ICArray(29)        ' Input Current 30 element array

AverageIC = 18         ' Average input (generator) current over last 60 seconds

Dim ICAArray(59)       ' array to hold 1 hours worth of average input currents

LoadCurrent = 12       ' Load current of caravan - can be supplied from the
                       ' batteries and the charging current from the charging
                       ' systems (solar, AC Mains or generator).
                       ' Read from pin 2

Dim LCArray(29)        ' Load Current 60 element array

AverageLC = 12         ' Average load current over last 60 seconds

Dim LCAArray(59)       ' array to hold 1 hours worth of average load currents

BatCurrent = 6         ' A calculated value; if positive, it represents a
                       ' charging current into the battery; if negative, it
                       ' represents a discharge current out of the battery.

AverageBC = 6          ' Average charge/discharge current over last min
                       ' calculated by AverageLC-AverageIC                       ' c

InputCurrentAH = 0     ' Input current as an amp hour calculated value -
                       ' this is an accumulating value over 24 hours
                       ' Previous days values are dummy values to allow
                       ' initial calculations

PreviousICAH = 0       ' yesterdays Input Current AHs

LoadCurrentAH = 0      ' Load current as an amp hour calculated value -
                       ' this is an accumulating value over 24 hours

PreviousLCAH = 0       ' yesterdays Load Current AHs

BatCurrentAH = 0       ' Bat charge (pos value) or discharge (neg value)
                       ' as an amp hour calculated value  -
                       ' this is an accumulating value over 24 hours

PreviousBCAH = 5       ' yesterdays Bat Current AHs

DayP2BCAH = -2          ' 5 day Bat Current in AHrs
DayP3BCAH = 3
DayP4BCAH = -5

LogArray$ = ""         ' String to use for plugging into log array

NewDataFlag = 0        ' Two second flag - set every 2 seconds by the
                       ' interrupt routine when new voltage and current
                       ' values are collected. Cleared as soon as data
                       ' processed in main loop

MinFlag = 0            ' Minute flag - set once every min at 00 seconds
                       ' by sec interrupt process and cleared by
                       ' min processing code.

HourFlag = 0           ' Hour Flag - set on the hour to enable
                       ' writing the 60 average readings in the log array
                       ' to be writtent to file

EODFlag = 0            ' End of Day Flag - set at midnight to enable
                       ' daily log file to be closed and new
                       ' log file created by LogFileCreate subroutine


'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' Carry out initialisation routines here - enable interrupt last

Entry:
Mode 3,7     ' 8 colour mode white printing
Colour 7,0   ' confirming white on black
Cls
Input "Enter 1 for debug mode, any other character for live mode ? ",a
If a <> 1 Then
  DEBUG=FALSE
Else
  DEBUG=TRUE
EndIf

Cls                 ' clear the screen ready to start program


' Call subroutine to display all static data
' Important note: This is to the VGA display ONLY. Older versions used the
' USB link to display on a PC - this is no longer supported.
Option USB Off    ' as we use pixel positioning, we don't want output to USB
DisplayStatic     ' all the labels, borders etc.

' Create Log File on program entry and then at start of new day, that is,
' straight after midnight
CreateLogFile     'call create log file subroutine


' Set up all initial values to default for orderly entry, primarily for
' use in debug mode.
PreviousICAH = 50
PreviousLCAH = 45
PreviousBCAH = 5
OldABV = 13
LFAPointer = 0

' Fill the battery volts array so % charge works
For Ax=0 To 29
  BVArray(Ax) = 13
Next Ax

' Wait until 00 seconds to start for clean entry.
MinTurnover=1
Do While MinTurnover <> 0
  MinTurnover=Val(Mid$(Time$,7,2))
Loop

' Enable timer tick interrupts to start data collection at 00 seconds
' - enable at 58 secs so first interrupt is at 00 secs
SetTick 2000,GetData   ' Every other second, get battery voltage and current
                       ' data through the GetData interrupt routine.

' end of initialisation routines

''''''''''''''''''''
' Main program loop
'
Main:
Do While NewDataFlag = 0   ' wait until interrupt provides fresh data
Loop

''''''''''''''''''''
' 2 second processing code
' Write out time and date to display
Print @(1,14)Time$ + "   " + Date$

' Push most recent values into array
For Ax = 29 To 1 Step -1
  BVArray(Ax) = BVArray(Ax-1)
Next Ax
BVArray(0) = BVolts

For Ax = 29 To 1 Step -1
  SVArray(Ax) = SVArray(Ax-1)
Next Ax
SVArray(0) = SVolts

For Ax = 29 To 1 Step -1
  ICArray(Ax) = ICArray(Ax-1)
Next Ax
ICArray(0) = InputCurrent

For Ax = 29 To 1 Step -1
  LCArray(Ax) = LCArray(Ax-1)
Next Ax
LCArray(0) = LoadCurrent

' Charge/Discharge Current is InputCurrent-LoadCurrent - if positive
'  it is a charging current into battery, if negative, it is a
'  discharging current out of battery
BatCurrent = InputCurrent-LoadCurrent

' Convert both input, output & battery instantaneous currents into Amp Hours
InputCurrentAH = InputCurrentAH + (InputCurrent/1800) ' use 2 sec interrupt
LoadCurrentAH = LoadCurrentAH + (LoadCurrent/1800)    ' rate for calculation
BatCurrentAH = InputCurrentAH-LoadCurrentAH

' Note: All AH accumulating totals are plugged into last day values
' then reset to zero at midnight except previous values which only
' change at midnight
' Set all of the variables to strings - some will have default
' values until after 2 minutes have passed

' Now update terminal display with all values
Print @(394,25)Format$(BatCharge,"%3.0f")+"%"  
Print @(111,45)Format$(BVolts,"%6.2fV")
Print @(111,56)Format$(SVolts,"%6.2fV")
Print @(111,67)Format$(InputCurrent,"%6.1fA")
Print @(111,78)Format$(LoadCurrent,"%6.1fA")
Print @(111,89)Format$(BatCurrent,"%+6.1fA")
Print @(312,45)Format$(AverageBV,"%6.2fV")
Print @(312,56)Format$(AverageSV,"%6.2fV")
Print @(376,149)Format$(Time2Discharge,"%4.0f min")
Print @(312,67)Format$(AverageIC,"%5.1fA")
Print @(312,78)Format$(AverageLC,"%5.1fA")
Print @(312,89)Format$(AverageBC,"%+5.1fA")
Print @(111,127)Format$(InputCurrentAH,"%6.1fahrs")
Print @(111,138)Format$(LoadCurrentAH,"%6.1fahrs")
Print @(111,149)Format$(BatCurrentAH,"%+6.1fahrs")
Print @(312,127)Format$(PreviousICAH,"%5.1fahrs")
Print @(312,138)Format$(PreviousLCAH,"%5.1fahrs")
Print @(312,149)Format$(PreviousBCAH,"%+5.1fahrs")
Print @(360,67)Format$(DayP2BCAH,"%+5.1fahrs")
Print @(360,78)Format$(DayP3BCAH,"%+5.1fahrs")
Print @(360,89)Format$(DayP4BCAH,"%+5.1fahrs")

NewDataFlag = 0      ' clear new data flag ready for next interrupt
'
' End of two second processing code, now check for minute, hour or end of
' day flags and process accordingly
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' Check for minute flag, calculate the average of the last 60 seconds of
' readings, plug them into the log file array at the log file array pointer
If MinFlag Then
  ' calculate average for last minute
  OldABV = AverageBV
  AverageBV = 0
  For Ax = 0 To 29   'don't forget only sampled every 2 secs
    AverageBV = AverageBV + BVArray(Ax)
  Next Ax
  AverageBV = AverageBV/30

  OldASV = AverageSV
  AverageSV = 0
  For Ax = 0 To 29
    AverageSV = AverageSV + SVArray(Ax)
  Next Ax
  AverageSV = AverageSV/30

  AverageIC = 0
  For Ax = 0 To 29
    AverageIC = AverageIC + ICArray(Ax)
  Next Ax
  AverageIC = AverageIC/30

  AverageLC = 0
  For Ax = 0 To 29
    AverageLC = AverageLC + LCArray(Ax)
  Next Ax
  AverageLC = AverageLC/30

  AverageBC = AverageIC - AverageLC
  ' Calculate battery change as percentage and fill charge state graph
  ' now calculate charge and paint capacity graph
  ' 12.1V equates to 30% charge
  BatCharge = Int((AverageBV - 11.75) * 100)
  If BatCharge < 30 Then BatCharge = 30
  If BatCharge > 100 Then BatCharge = 100
  FillBarGraph(BatCharge)

  ' Calculate time to 30% discharge of battery
  DeltaBV = OldABV - AverageBV
  If Sgn(DeltaBV) = +1 Then
       DVRate = BVArray(0)         ' set initial values for rate of discharge
    Time2Discharge = 0             ' and time to discharge
    Do While DVRate > 12.1         ' 12.1 volts equates to 30% discharge
      DVRate = DVRate - DeltaBV    ' loop through calculating the amount
                                   ' of time to get the current voltage
                                   ' down to the 30% discharge point
      Time2Discharge = Time2Discharge + 1 ' increment minutes to discharge count
    Loop
  Else
    Print @(376,149)"Charging "   ' if current voltage is greater than
  EndIf                            ' previous minute


   BVAArray(LFAPointer) = AverageBV  ' update average arrays
   SVAArray(LFAPointer) = AverageSV
   ICAArray(LFAPointer) = AverageIC
   LCAArray(LFAPointer) = AverageLC

  LFAPointer = LFAPointer + 1          ' LFA pointer reset by eod routine

' Now paint out running graph
' - first work out the current minute hour combo in minutes
  rg_mins = (((Val(Mid$(Time$,1,2)) Mod 6)*60))+(Val(Mid$(Time$,4,2)))
  rg_mins = rg_mins + 30               ' offset to start of graph
  rg_abv = 413 - Int((AverageBV-7.5) * 20)
    if rg_abv > 413 then rg_abv = 413
  rg_asv = 413 - Int((AverageSV-7.5) * 20)
    if rg_asv > 413 then rg_asv = 413
    if rg_asv < 163 then rg_asv = 163
  rg_aic = 413 - Int(AverageIC * 5)
  rg_alc = 413 - Int(AverageLC * 5)
  If (Sgn(AverageBC) = +1) Then
    rg_abc = 413 - Int(AverageBC * 5)
  Else
    rg_abc = 413 + Int(AverageBC * 5)
  EndIf
  ' now paint running graph points
  For rg_y = 163 To 413 Step 1
    If rg_abc = rg_y Then
      If (Sgn(AverageBC) = +1) Then
        Pixel(rg_mins,rg_y) = 2  ' 2=green
      Else
        Pixel(rg_mins,rg_y) = 4  ' 4=red
      EndIf
    Else
      Pixel(rg_mins,rg_y) = 0    ' 0=black
    EndIf
    If rg_abv = rg_y Then
      Pixel(rg_mins,rg_y) = 7  ' 7=white
    EndIf
    If rg_asv = rg_y Then
      Pixel(rg_mins,rg_y) = 1  ' 1=blue
    EndIf
    If rg_aic = rg_y Then
      Pixel(rg_mins,rg_y) = 6  ' 6=yellow
    EndIf
    If rg_alc = rg_y Then
      Pixel(rg_mins,rg_y) = 5  ' 5=magenta
    EndIf
  Next rg_y
  For rg_y = 163 To 413 Step 1
    Pixel(rg_mins+1,rg_y) = 0
    Pixel(rg_mins+2,rg_y) = 0
    Pixel(rg_mins+3,rg_y) = 0
  Next rg_y

  MinFlag = 0                          ' reset the min flag
EndIf
' End of minute processing
''''''''''''''''''''''''''''
' Check for hour flag - if true then append log array out to file
' (we want to limit log file writes to 24 per day to keep
' SD card writes as low as possible).
' - use the average arrays
If HourFlag Then          ' set each hour in interrupt routine
  LogArray$ = ""
  For x=0 To 59 Step 1    ' get the average values from each average array
    LogArray$ = ""
    LogArray$ = LogArray$+Format$(BVAArray(x),"%+6.2fV")+Chr$(44)
    LogArray$ = LogArray$+Format$(SVAArray(x),"%+6.2fV")+Chr$(44)
    LogArray$ = LogArray$+Format$(ICAArray(x),"%+5.1fA")+Chr$(44)
    LogArray$ = LogArray$+Format$(LCAArray(x),"%+5.1fA")
    Print #1,LogArray$
  Next x
  LFAPointer = 0              ' reset array pointer
  HourFlag = 0                ' clear hour flag until next hour
EndIf
' End of hour processing code
''''''''''''''''''''''''''''''
' Check for end of day - if true then close log file and create new file
' for new day.
If EODFlag Then
  Close #1                ' close out last days log file
  ' ... Now Save all the amp hour readings and reset ready for next day
  PreviousICAH = InputCurrentAH
  InputCurrentAH = 0
  PreviousLCAH = LoadCurrentAH
  LoadCurrentAH = 0

  DayP4BCAH = DayP3BCAH
  DayP3BCAH = DayP2BCAH
  DayP2BCAH = PreviousBCAH
  PreviousBCAH = BatCurrentAH
  BatCurrentAH = 0

  CreateLogFile           ' create new log file with todays date
  EODFlag = 0             ' clear end of day flag until next midnight
EndIf
' End of day processing code

'''''''''''''''''''''''''''''
GoTo Main                 ' back to Main ready for next interrupt
'
' End of main program loop
'''''''''''''''''''''''''''''

'''''''''''''''''''''''''''''
' Interrupt Routine Initialisation and Handling

GetData:               ' Timer Interrupt handler
                       ' Analogue input pins
  SetPin 1,1           ' Pin 4 - battery voltage - battery divided by 5
  SetPin 2,1           ' Pin 1 - input current shunt - 0 to 2.5V = 0 to 50A
  SetPin 4,1           ' Pin 2 - load current shunt - 0 to 2.5V = 0 to 50A
  SetPin 3,1           ' Pin 3 - solar volts - panel volts divided by 7

' Get the battery voltage
If Not Debug Then
  ic=0
  BVolts = 0
  GetPin4 = 0
  For ic = 1 To 10
    GetPin4 = Pin(4)
    BVolts = BVolts + GetPin4
  Next ic
  BVolts = BVolts/10          ' Average out Battery voltage
  BVolts = BVolts*6           ' 15 battery volts = 2.5 MM volts

Else
  BVolts=AverageBV - (Rnd(1)/20)  ' this can be all manipulated to simulate
  If BVolts < 11.5 Then BVolts = 11.5   ' whatever values you want
EndIf

' Get the Solar Array voltage
If Not Debug Then
  ic=0
  SVolts=0
  GetPin3 = 0
  For ic = 1 To 10
    GetPin3 = Pin(3)
    SVolts = SVolts + GetPin3
  Next ic
  SVolts = SVolts/10          ' same as for BVolts
  SVolts = SVolts*10          ' 25 solar volts = 2.5 MM volts

Else
 SVolts=AverageSV -(Rnd(1)/20)
 If SVolts < 0 Then SVolts = 0
EndIf

' Get the input current as a voltage
If Not Debug Then
  ic=0
  InputCurrent = 0
  GetPin1 = 0
  For ic = 1 To 10
       GetPin1 = Pin(1)
       InputCurrent = InputCurrent + GetPin1
  Next ic
  InputCurrent = InputCurrent/10   ' Average out voltage representing
                                            ' input current
  InputCurrent = InputCurrent*20 ' Correct for opamp amplification - 2.5V by 20
                               ' now gives a value equating to ratio 50mV = 50A
                               ' Apply calculated correction factor

Else
 InputCurrent=AverageIC-Rnd(1/30)
 If InputCurrent < 0 Then InputCurrent = 0
EndIf

' Get the load current as a voltage
If Not Debug Then
  ic=0
  LoadCurrent = 0
  GetPin2 = 0
  For ic = 1 To 10
       GetPin2 = Pin(2)
       LoadCurrent = LoadCurrent + GetPin2
  Next ic
  LoadCurrent = LoadCurrent/10 ' Average out voltage representing
                                        ' load current
  LoadCurrent = LoadCurrent*20 ' Correct for opamp amplification - 2.5V by 20
                               ' now gives a value equating to ratio 50mV = 50A

Else
 LoadCurrent=AverageLC-(Rnd(1)/40)      ' load current decreases at lesser
 If LoadCurrent < 0 Then LoadCurrent = 0 ' input current rate to simulate
EndIf                                    ' loss of sun at evening

' Get the current time, extract the minutes component
' check for 00 secs
If Mid$(Time$,7,2) = "00" Then
  MinFlag = 1
EndIf

' Get the current time, test for hour turnover and
' set HourFlag to 1 to initiate logging collected data to file
CurrentMins$ = Mid$(Time$,4,2) ' extract minutes
If Mid$(Time$,7,2) = "00" And Mid$(Time$,4,2) = "00" Then
  HourFlag = 1
EndIf

' Get the current time, test for midnight and
' set EODFlag to 1 to initiate logging collected data to file
CurrentHrs$ = Mid$(Time$,1,2)  ' extract hours
If Mid$(Time$,7,2)="00" And Mid$(Time$,4,2)="00" And Mid$(Time$,1,2)="00" Then
  EODFlag = 1
EndIf

' Increment seconds counter
' SecsSinceStart = SecsSinceStart + 1
NewDataFlag = 1                ' set to indicate fresh data

IReturn                        ' Returns with the following values:-
                               ' BVolts = 0 to 15 equalling battery volts
                               ' SVolts = 0 to 25 equalling solar Array volts
                               ' InputCurrent=0 to 50 equalling 0 to 50Amps
                               ' LoadCurrent=0 to 50 equalling 0 to 50Amps
                               ' Note: All values have had the correction
                               ' factor applied.
                               ' EODFlag set at midnight
'End of Interrupt code
'''''''''''''''''''''''
' Defined Subroutines
' - TPrintStatus - update status area
' - Create Data File - create new log file at start and midnight
' - TermDisplayStatic - write static terminal display data

'''''''''''''''''''''''
' TPrintStatus Status$ - a subroutine to clear staus area and print a status
'  message - expects a string to print
Sub PrintStatus Status$
  Print @(348,14)"            "
  Print @(348,14)Status$
End Sub

'''''''''''''''''''''''
' Set up terminal display with static data (field names etc.)
' - uses the PRINT @ to print at defined positions
' - also static parts of bar graph and running graph
' Note:  If you want to update the bmp you will need to enable this
'  subroutine and run once to save the bmp
Sub DisplayStatic
  Cls
  Print @(150,1)"CARAVAN POWER MONITOR V0.99"
  Print @(1,25)"Instantaneous Readings"
  Print @(198,25)"Minute Average Readings"
  Line(1,36)-(158,36)
  Line(198,36)-(350,36)
  Print @(384,1)"Battery"
  Print @(1,45)"Battery Volts     ="
  Print @(198,45)"Battery Volts     ="
  Print @(387,12)"Charge"
  Print @(1,56)"Solar Array Volts ="
  Print @(198,56)"Solar Array Volts ="
  Print @(1,67)"Input Current     ="
  Print @(198,67)"Input Current     ="
  Print @(1,78)"Load Current      ="
  Print @(198,78)"Load Current      ="
  Print @(1,89)"Bat Chg/Dis Curr  ="
  Print @(198,89)"Bat Chg/Dis Curr  ="
  Print @(1,108)"Daily Accumulating Readings"
  Print @(198,108)"Previous Days Total Readings"
  Print @(369,45)"Bat Charge"
  Print @(366,56)"Last 3 Days"
  Line(1,120)-(170,120)
  Line(198,120)-(362,120)
  Print @(1,127)"Input Charge      ="
  Print @(198,127)"Input Charge      ="
  Print @(384,127)"Time to"
  Print @(1,138)"Load Discharge    ="
  Print @(198,138)"Load Discharge    ="
  Print @(400,138)"30%"
  Print @(1,149)"Battery Chg/DisChg="
  Print @(198,149)"Battery Chg/DisChg="
  Line(0,162)-(479,431),1,B
  Colour 7,0
  Print @(424,164)"Battery"
  Print @(430,174)"Volts"
  Colour 1,0
  Print @(430,194)"Solar"
  Print @(430,204)"Volts"
  Colour 6,0
  Print @(430,224)"Input"
  Print @(433,234)"Amps"
  Colour 5,0
  Print @(433,254)"Load"
  Print @(433,264)"Amps"
  Colour 4,0
  Print @(427,284)"Battery"
  Print @(424,294)"Amps Out"
  Colour 2,0
  Print @(427,314)"Battery"
  Print @(427,324)"Amps In"
  Colour 7,0
  volts=20
  amps = 50
  For ry =163 to 403 step 10
    Print @(400,ry)Format$(amps,"%2.0fA")
    amps=amps-2
  next ry
  For ry =163 to 403 step 20
    Print @(2,ry)Format$(volts,"%2.0fV")
    volts=volts-1
  next ry
  Line(29,163)-(29,413)
  Line(391,163)-(391,413)
  For yrg = 163 To 413 Step 10 'major ticks both sides
    Pixel(28,yrg)=7
    Pixel(27,yrg)=7
    Pixel(392,yrg)=7
    Pixel(393,yrg)=7
  Next yrg
  For yrg = 163 To 413 Step 20  'minor ticks on volts side
    Pixel(27,yrg)=7
  Next yrg
 
  Line(30,414)-(390,414)
  For xrg = 30 To 390 Step 5
    Pixel(xrg,415)=7
  Next xrg
  For xrg = 30 To 390 Step 15
    Pixel(xrg,415)=7
    Pixel(xrg,416)=7
  Next xrg
  For xrg = 30 To 390 Step 30
    Pixel(xrg,415)=7
    Pixel(xrg,416)=7
    Pixel(xrg,417)=7
  Next xrg
  For xrg = 30 To 390 Step 60
    Pixel(xrg,415)=7
    Pixel(xrg,416)=7
    Pixel(xrg,417)=7
    Pixel(xrg,418)=7
  Next xrg
  x_label=1
  for bx = 88 to 388 step 60
    Print @(bx,418)chr$(x_label)    ' running graph x labels - hours -
    x_label=x_label+1     ' so we display a running 6 hour window
  next bx

' Bar Graph - Draw Static
' top left x=432, y=0
' bottom right x=479, y=162
' To fill, start at x=434 thru 443
' and y=145 back up to y=15
' This gives 130 steps. In turn this
' equates to 2 steps per percentage for the
' range of 35% to 100%
' - use BarGraphFill to enter battery charge data
'
  Local xtl,ytl,xbr,ybr,percent,xpos,ypos,y''

  xtl=432:ytl=0:xbr=xtl+47:ybr=ytl+162
'  First draw the box and values
  Line(xtl,ytl)-(xbr,ybr),1,B
  percent=100
  Print @(xtl+3,ytl+2)"Battery"
  Print @(xtl+13,ytl+13)100
  Print @(xtl+40,ytl+13)"%"
  percent=percent-5
  For y=ytl+22 To 142 Step 10
    Print @(xtl+19,y)percent
    Print @(xtl+40,y)"%"
    percent = percent-5
  Next y
End Sub
''''''''''''''''''''''
' Create Log File with todays date as name
'
Sub CreateLogFile
  LogFileName$="B:"+Mid$(Date$,9,2)+Mid$(Date$,4,2)+Left$(Date$,2)+".CSV"
  Open LogFileName$ For Output As #1
  Print #1, "Battery Voltage";Chr$(44);
  Print #1, "Solar Array Voltage";Chr$(44);
  Print #1, "Input Current";Chr$(44);
  Print #1, "Load Current"
End Sub

''''''''''''''''''''''
' Bar Graph - Fill
' top left x=432, y=0
' bottom right x=479, y=162
' To fill, start at x=434 thru 443
' and y=150 back up to y=12
' This gives 138 steps. In turn this
' equates to 1.97 steps per percentage for the
' range of 30% to 100%
' Enter with actual battery charge as a percent
'
Sub FillBarGraph(BatCharge)

Local xtl,ytl,bat_chg,xpos,ypos,yper,xright,xleft,ybot,ytop

bat_chg=BatCharge
xtl=432:ytl=0:xbr=xtl+47:ybr=ytl+162

' Now fill the bar graph
If bat_chg < 31 Then
  bat_chg=30
  Colour 4,7                            ' red on white
  Print @(xtl + 4,ytl + 151,2)"DANGER"
Else
  Colour 2,7                            ' green on white
  Print @(xtl + 4,ytl + 151,2)"  OK  "
EndIf
Colour 7,0                              ' back to white on black
If bat_chg > 100 Then bat_chg=100
ytop=12
yper=Int(((100 - bat_chg) * 1.97) + 12)
ybot=150
xleft=437
xright=446
For ypos=ybot To ytop Step -1
  If yper < ypos Then
    For xpos=xleft To xright Step 1
      Pixel(xpos,ypos)=2
    Next xpos
  Else
    For xpos=xleft To xright Step 1
      Pixel(xpos,ypos)=0
    Next xpos
  EndIf
Next ypos
End Sub




















